@@ -1,6 +1,6 @@
 # -*- Coding: utf-8 -*-
 
-# Copyright 2014 Red Hat, Inc.
+# Copyright 2014-2015 Red Hat, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -21,15 +21,13 @@ import os_net_config
 from os_net_config import objects
 from os_net_config import utils
 
-from os_net_config.openstack.common import processutils
-
 
 logger = logging.getLogger(__name__)
 
 
 # TODO(?): should move to interfaces.d
-def _network_config_path():
-    return "/etc/network/interfaces"
+def _network_config_path(prefix=''):
+    return prefix + "/etc/network/interfaces"
 
 
 class ENINetConfig(os_net_config.NetConfig):
@@ -39,28 +37,34 @@ class ENINetConfig(os_net_config.NetConfig):
        /etc/network/interfaces format.
     """
 
-    def __init__(self):
+    def __init__(self, noop=False, root_dir=''):
+        super(ENINetConfig, self).__init__(noop, root_dir)
         self.interfaces = {}
         self.routes = {}
         self.bridges = {}
         logger.info('ENI net config provider created.')
 
-    def _add_common(self, interface, static_addr=None):
+    def _add_common(self, interface, static_addr=None, ip_version=4):
 
         ovs_extra = []
         data = ""
         address_data = ""
         if static_addr:
             address_data += "    address %s\n" % static_addr.ip
-            address_data += "    netmask %s\n" % static_addr.netmask
+            if ip_version == 6:
+                address_data += "    netmask %s\n" % static_addr.prefixlen
+            else:
+                address_data += "    netmask %s\n" % static_addr.netmask
         else:
             v4_addresses = interface.v4_addresses()
             if v4_addresses:
-                data += self._add_common(interface, v4_addresses[0])
+                for v4_address in v4_addresses:
+                    data += self._add_common(interface, v4_address)
 
             v6_addresses = interface.v6_addresses()
             if v6_addresses:
-                data += self._add_common(interface, v6_addresses[0])
+                for v6_address in v6_addresses:
+                    data += self._add_common(interface, v6_address, 6)
 
             if data:
                 return data
@@ -123,12 +127,18 @@ class ENINetConfig(os_net_config.NetConfig):
             data += address_data
             data += "    vlan-raw-device %s\n" % interface.device
         else:
-            data += "auto %s\n" % interface.name
+            if isinstance(interface, objects.Interface) and interface.hotplug:
+                data += "allow-hotplug %s\n" % interface.name
+            else:
+                data += "auto %s\n" % interface.name
             data += _iface
             data += address_data
-        if interface.mtu != 1500:
+        if interface.mtu:
             data += "    mtu %i\n" % interface.mtu
 
+        if interface.hwaddr:
+            raise NotImplementedError("hwaddr is not implemented.")
+
         if ovs_extra:
             data += "    ovs_extra %s\n" % " -- ".join(ovs_extra)
 
@@ -174,61 +184,64 @@ class ENINetConfig(os_net_config.NetConfig):
         logger.info('adding custom route for interface: %s' % interface_name)
         data = ""
         for route in routes:
+            options = ""
+            if route.route_options:
+                options = " %s" % (route.route_options)
             if route.default and not route.ip_netmask:
                 rt = netaddr.IPNetwork("0.0.0.0/0")
             else:
                 rt = netaddr.IPNetwork(route.ip_netmask)
-            data += "up route add -net %s netmask %s gw %s\n" % (
-                    str(rt.ip), str(rt.netmask), route.next_hop)
-            data += "down route del -net %s netmask %s gw %s\n" % (
-                    str(rt.ip), str(rt.netmask), route.next_hop)
+            data += "up route add -net %s netmask %s gw %s%s\n" % (
+                    str(rt.ip), str(rt.netmask), route.next_hop, options)
+            data += "down route del -net %s netmask %s gw %s%s\n" % (
+                    str(rt.ip), str(rt.netmask), route.next_hop, options)
         self.routes[interface_name] = data
         logger.debug('route data: %s' % self.routes[interface_name])
 
-    def apply(self, noop=False, cleanup=False):
+    def apply(self, cleanup=False, activate=True):
         """Apply the network configuration.
 
-        :param noop: A boolean which indicates whether this is a no-op.
+        :param cleanup: A boolean which indicates whether any undefined
+            (existing but not present in the object model) interface
+            should be disabled and deleted.
+        :param activate: A boolean which indicates if the config should
+            be activated by stopping/starting interfaces
         :returns: a dict of the format: filename/data which contains info
             for each file that was changed (or would be changed if in --noop
             mode).
+        Note the noop mode is set via the constructor noop boolean
         """
         new_config = ""
 
         # write out bridges first. This ensures that an ifup -a
         # on reboot brings them up first
-        for bridge_name, bridge_data in self.bridges.iteritems():
+        for bridge_name, bridge_data in self.bridges.items():
             route_data = self.routes.get(bridge_name)
             bridge_data += (route_data or '')
             new_config += bridge_data
 
-        for interface_name, iface_data in self.interfaces.iteritems():
+        for interface_name, iface_data in self.interfaces.items():
             route_data = self.routes.get(interface_name)
             iface_data += (route_data or '')
             new_config += iface_data
 
-        if (utils.diff(_network_config_path(), new_config)):
-            if noop:
-                return {"/etc/network/interfaces": new_config}
-            for interface in self.interfaces.keys():
-                logger.info('running ifdown on interface: %s' % interface)
-                processutils.execute('/sbin/ifdown', interface,
-                                     check_exit_code=False)
+        if utils.diff(_network_config_path(self.root_dir), new_config):
+            if activate:
+                for interface in self.interfaces.keys():
+                    self.ifdown(interface)
 
-            for bridge in self.bridges.keys():
-                logger.info('running ifdown on bridge: %s' % bridge)
-                processutils.execute('/sbin/ifdown', bridge,
-                                     check_exit_code=False)
+                for bridge in self.bridges.keys():
+                    self.ifdown(bridge, iftype='bridge')
 
-            logger.info('writing config file')
-            utils.write_config(_network_config_path(), new_config)
+            self.write_config(_network_config_path(self.root_dir), new_config)
 
-            for bridge in self.bridges.keys():
-                logger.info('running ifup on bridge: %s' % bridge)
-                processutils.execute('/sbin/ifup', bridge)
+            if activate:
+                for bridge in self.bridges.keys():
+                    self.ifup(bridge, iftype='bridge')
 
-            for interface in self.interfaces.keys():
-                logger.info('running ifup on interface: %s' % interface)
-                processutils.execute('/sbin/ifup', interface)
+                for interface in self.interfaces.keys():
+                    self.ifup(interface)
         else:
             logger.info('No interface changes are required.')
+
+        return {_network_config_path(self.root_dir): new_config}
